@@ -63,6 +63,8 @@ class Cell : public MWAWCell
 public:
   /// constructor
   Cell() : m_content(), m_noteId(0) { }
+  //! destructor
+  virtual ~Cell();
   //! returns true if the cell do contain any content
   bool isEmpty() const
   {
@@ -134,6 +136,10 @@ protected:
   }
 };
 
+Cell::~Cell()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the state of a MsWksSSParser
 struct State {
@@ -288,7 +294,7 @@ void MsWksSSParser::sendNote(int noteId)
   while (1) {
     long pos=input->tell();
     if (input->isEnd() || pos>=entry.end()) return;
-    int c = (int) input->readULong(1);
+    int c = static_cast<int>(input->readULong(1));
     switch (c) {
     case 0x9:
       listener->insertTab();
@@ -322,10 +328,10 @@ void MsWksSSParser::sendNote(int noteId)
         }
       }
       else if (c <= 0x1f) {
-        MWAW_DEBUG_MSG(("MsWksSSParser::sendNote: find char=%x\n", (unsigned int)c));
+        MWAW_DEBUG_MSG(("MsWksSSParser::sendNote: find char=%x\n", static_cast<unsigned int>(c)));
       }
       else
-        listener->insertCharacter((unsigned char)c, input, entry.end());
+        listener->insertCharacter(static_cast<unsigned char>(c), input, entry.end());
       break;
     }
 
@@ -407,7 +413,7 @@ bool MsWksSSParser::createZones()
     MWAW_DEBUG_MSG(("MsWksSSParser::createZones: find some unexpected data\n"));
     pos = input->tell();
     MsWksDocument::Zone unknown;
-    if (!m_document->readZone(unknown)) {
+    if (!m_document->readZone(unknown) || input->tell() <= pos) {
       ascFile.addPos(pos);
       ascFile.addNote("Entries(End)");
       ascFile.addPos(pos+100);
@@ -450,32 +456,32 @@ bool MsWksSSParser::readSSheetZone()
 
   int N[2]; // max col, max row
   for (int i = 0; i < 2; i++)
-    N[i] = (int) input->readLong(2);
+    N[i] = static_cast<int>(input->readLong(2));
   f << "maxCol=" << N[0] << ","  << "maxRow=" << N[1] << ",";
 
   f << "unk0=" << std::hex << input->readULong(4) << std::dec << ","; // big number
 
   int zoneSize[3];
-  zoneSize[0] = (int) input->readLong(2);
+  zoneSize[0] = static_cast<int>(input->readLong(2));
   if (zoneSize[0]) f << "zone[SheetF]=" << std::hex << zoneSize[0] << std::dec << ",";
-  int numPageBreak = (int) input->readLong(2);
+  int numPageBreak = static_cast<int>(input->readLong(2));
   if (numPageBreak) f << "nPgBreak=" << numPageBreak << ",";
-  int numColBreak = (int) input->readLong(2);
+  int numColBreak = static_cast<int>(input->readLong(2));
   if (numColBreak) f << "nColBreak=" << numColBreak << ",";
-  zoneSize[1]= (int) input->readLong(2);
+  zoneSize[1]= static_cast<int>(input->readLong(2));
   if (zoneSize[1]) f << "zone[DocInfo]=" << zoneSize[1] << ",";
   f << "unk1=" << std::hex << input->readULong(4) << std::dec << ","; // big number
   f << std::dec;
 
   int val;
   for (int i = 0; i < 2; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << "w" << i << "=" << val << ",";
   }
   f << "v2=["; // [_,_,_,_,_,_,] or [_,1,3,_,1,3,] or [_,5,5,_,4,2,]
   for (int i = 0; i < 6; i++) {
-    val = (int) input->readLong(1);
+    val = static_cast<int>(input->readLong(1));
     if (val)
       f << val << ",";
     else
@@ -484,7 +490,7 @@ bool MsWksSSParser::readSSheetZone()
   f << "]";
   f << ", v3=["; // [_,_,_,_,] or [_,_,_,1,] or [_,_,_,11,]
   for (int i = 0; i < 4; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << val << ",";
     else
@@ -494,7 +500,7 @@ bool MsWksSSParser::readSSheetZone()
 
   f << ", v4=[" << std::hex;
   for (int i = 0; i < 4; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << val << ",";
     else
@@ -514,12 +520,12 @@ bool MsWksSSParser::readSSheetZone()
   int prevW = -1;
   bool ok = true;
   for (int i = 0; i < 257; i++) {
-    int col = (int) input->readULong(2);
+    int col = static_cast<int>(input->readULong(2));
     if ((col & 0xFFF) != i+1) {
       ok = false;
       break;
     }
-    int w = (int) input->readLong(2);
+    int w = static_cast<int>(input->readLong(2));
     if (w > 0) {
       if (prevW > w && i < 200) {
         ok = false;
@@ -571,8 +577,8 @@ bool MsWksSSParser::readSSheetZone()
   for (int i=0; i < 8; ++i) {
     // the chart names in v2, can we have more than 8 charts ?
     long cPos=input->tell();
-    val = (int) input->readLong(2);
-    int sSz=(int) input->readULong(1);
+    val = static_cast<int>(input->readLong(2));
+    int sSz=static_cast<int>(input->readULong(1));
     if (!sSz || sSz>32-7) {
       input->seek(cPos, librevenge::RVNG_SEEK_SET);
       break;
@@ -582,7 +588,7 @@ bool MsWksSSParser::readSSheetZone()
     if (val) f << "unkn=" << val << ",";
     std::string name("");
     for (int c=0; c<sSz; ++c)
-      name += (char) input->readULong(1);
+      name += char(input->readULong(1));
     f << name << ",";
     if ((sSz%2)==0)   input->seek(1, librevenge::RVNG_SEEK_CUR);
     f << "id=" << std::hex << input->readULong(4) << std::dec << "],";
@@ -590,7 +596,7 @@ bool MsWksSSParser::readSSheetZone()
   }
   int numRemains=int(endPos-input->tell())/2;
   for (int i = 0; i < numRemains; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   ascFile.addNote(f.str().c_str());
@@ -608,7 +614,7 @@ bool MsWksSSParser::readSSheetZone()
 
   f << "SSheetD:";
   for (int i = 0; i < 5; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (i==3) {
       if (val==44) continue;
       f << "unkn=" << val << ",";
@@ -625,9 +631,9 @@ bool MsWksSSParser::readSSheetZone()
   for (int i = 0; i < 2; i++) {
     f << ", g" << i << "="  << std::hex << input->readLong(2) << std::dec;
   }
-  int fId = (int) input->readLong(2);
-  int fSize = (int) input->readLong(2);
-  sheet.m_font = MWAWFont(fId, (float) fSize);
+  int fId = static_cast<int>(input->readLong(2));
+  int fSize = static_cast<int>(input->readLong(2));
+  sheet.m_font = MWAWFont(fId, float(fSize));
   f << ", [" <<  sheet.m_font.getDebugString(getParserState()->m_fontConverter) << "]";
   // , [0,{33|255},] ?
   f << ", unk=[" << std::dec;
@@ -636,7 +642,7 @@ bool MsWksSSParser::readSSheetZone()
   f << "]";
   int numRemain=int(sheetDebPos+m_document->getLengthOfFileHeader3()-input->tell())/2;
   for (int i = 0; i < numRemain; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   ascFile.addNote(f.str().c_str());
@@ -662,7 +668,7 @@ bool MsWksSSParser::readSSheetZone()
       ascFile.addNote(f.str().c_str());
       continue;
     }
-    long endRow = posRow+(long)length+4;
+    long endRow = posRow+long(length)+4;
 
     if (!input->checkPosition(endRow)) {
       MWAW_DEBUG_MSG(("MsWksSSParser::readSSheetZone:Can not read part E(row)\n"));
@@ -675,10 +681,10 @@ bool MsWksSSParser::readSSheetZone()
     f << "SSheetE(Row" << std::dec << cellPos[1]+1 << "): ";
     while (input->tell() < endRow-1) {
       pos = input->tell();
-      int sz = (int) input->readULong(1);
+      int sz = static_cast<int>(input->readULong(1));
 
       if (sz == 0xFE) { // skip next cell
-        int numC = (int) input->readULong(1);
+        int numC = static_cast<int>(input->readULong(1));
         f2.str("");
         f2 << "SSheetE:skipC=" << numC;
         cellPos[0]+=numC;
@@ -710,7 +716,7 @@ bool MsWksSSParser::readSSheetZone()
       return false;
     }
     ascFile.addDelimiter(endRow-1,'|');
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val == 0) f << "*";
     else if (val != 0xFF)
       f << "##end=" << std::hex << val << std::dec;
@@ -738,10 +744,10 @@ bool MsWksSSParser::readSSheetZone()
   f.str("");
   f << "SSheetF[B]:" << std::dec;
   for (int i = 0; i < 4; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
-  val = (int) input->readULong(2);
+  val = static_cast<int>(input->readULong(2));
   if (val) f << std::hex << "fl=" << val << "," << std::dec;
   ascFile.addPos(pos);
   ascFile.addNote(f.str().c_str());
@@ -751,10 +757,10 @@ bool MsWksSSParser::readSSheetZone()
     f.str("");
     f << "SSheetG:";
     if (numPageBreak > 0) {
-      sheet.m_listPageBreaks.resize((size_t) numPageBreak);
+      sheet.m_listPageBreaks.resize(size_t(numPageBreak));
       f << "pgBreak=[";
-      for (size_t pgBreak = 0; pgBreak < (size_t) numPageBreak; pgBreak++) {
-        val = (int) input->readULong(2);
+      for (size_t pgBreak = 0; pgBreak < size_t(numPageBreak); pgBreak++) {
+        val = static_cast<int>(input->readULong(2));
         sheet.m_listPageBreaks[pgBreak] = val;
         f << val << ",";
       }
@@ -805,7 +811,7 @@ bool MsWksSSParser::readSSheetZone()
   int numNote = 0;
   while (1) {
     pos = input->tell();
-    int idNote = (int) input->readLong(2);
+    int idNote = static_cast<int>(input->readLong(2));
     if (idNote == 0) break;
     if (idNote != ++numNote) {
       input->seek(pos, librevenge::RVNG_SEEK_CUR);
@@ -813,13 +819,13 @@ bool MsWksSSParser::readSSheetZone()
       return false;
     }
     // a pascal string
-    int pSz = (int) input->readLong(2);
+    int pSz = static_cast<int>(input->readLong(2));
     MWAWEntry note;
     note.setBegin(input->tell());
     note.setLength(pSz);
     std::string str="";
     for (int s = 0; s < pSz; s++)
-      str += (char) input->readLong(1);
+      str += char(input->readLong(1));
     if (input->tell() != pos+4+pSz) {
       input->seek(pos, librevenge::RVNG_SEEK_CUR);
       MWAW_DEBUG_MSG(("MsWksSSParser::readSSheetZone:Pb when reading a note\n"));
@@ -856,7 +862,7 @@ bool MsWksSSParser::readCell(int sz, MWAWVec2i const &cellPos, MsWksSSParserInte
   input->seek(endPos, librevenge::RVNG_SEEK_SET);
   int fl[5];
   for (int i  = 0; i < 5; i++)
-    fl[i] = (int) input->readULong(1);
+    fl[i] = static_cast<int>(input->readULong(1));
   MWAWCell::Format format;
   MWAWCellContent &content=cell.m_content;
   int style = 0, align = 0, subformat = 0;
@@ -1038,9 +1044,11 @@ bool MsWksSSParser::readCell(int sz, MWAWVec2i const &cellPos, MsWksSSParserInte
     case MWAWCell::F_UNKNOWN:
       MWAW_DEBUG_MSG(("MsWksSSParser::readCell: unexpected format\n"));
       break;
+#if !defined(__clang__)
     default:
       f << ",subform==##unkn" << subformat;
       break;
+#endif
     }
   }
   long pos = debPos;
@@ -1055,7 +1063,7 @@ bool MsWksSSParser::readCell(int sz, MWAWVec2i const &cellPos, MsWksSSParserInte
       if (content.m_textEntry.length()>=0) {
         std::string text("");
         for (long i=0; i <content.m_textEntry.length(); ++i)
-          text += (char) input->readULong(1);
+          text += char(input->readULong(1));
         f << text << ",";
       }
       else
@@ -1088,7 +1096,9 @@ bool MsWksSSParser::readCell(int sz, MWAWVec2i const &cellPos, MsWksSSParserInte
     case MWAWCellContent::C_NONE:
       break;
     case MWAWCellContent::C_UNKNOWN:
+#if !defined(__clang__)
     default:
+#endif
       ok = false;
     }
     if (!ok)
@@ -1160,7 +1170,7 @@ bool MsWksSSParser::sendSpreadsheet()
       listener->setFont(cell.isFontSet() ? cell.getFont() : sheet.m_font);
       input->seek(cell.m_content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
       while (!input->isEnd() && input->tell()<cell.m_content.m_textEntry.end()) {
-        unsigned char c=(unsigned char) input->readULong(1);
+        unsigned char c=static_cast<unsigned char>(input->readULong(1));
         if (c==0xd)
           listener->insertEOL();
         else
